import sys, random

USAGE = 'Usage: emitdiag.py [pwmLen] [lenMin] [lenMax] [nSeq] [seed] [seqFile]'

AACODE = 'ACDEFGHIKLMNPQRSTVWY'
aacode = AACODE.lower()

def sum(sequence):
    'This function should be removed under Python 2.3+'
    total = sequence[0]
    for x in sequence[1:]: total = total + x
    return total

def createDiagPwm(pwmLen):
    step = 2 #20 / pwmLen
    pwm = []
    for i in xrange(pwmLen):
        pdf = [1.0] * 20
        pdf[i * step] = 70.0
        for k in xrange(i * step + 1, (i+1) * step): pdf[k] = 12.0 / (step-1)
        total = sum(pdf)
        for k in xrange(20): pdf[k] = pdf[k] / total
        pwm.append(pdf)
    return pwm

def pdf2cdf(pdf):
    cdf = [pdf[0]]
    for p in pdf[1:]: cdf.append(cdf[-1] + p)
    return cdf

# This should be in a utility module, or replaced by numpy.random.multinomial
def randomAA(cdf):
    'Generate a random index based on a given probability simplex'
    r = random.random()
    for i in xrange(len(cdf)):
        if cdf[i] > r: return AACODE[i]
    raise Exception('Illegal multinomial CDF')

def emit(pwmCdf, lenMin, lenMax):
    seqLen = random.randint(lenMin, lenMax)
    st = random.randint(0, seqLen - len(pwmCdf))
    bgInstance = ''.join([random.choice(aacode) \
                          for i in xrange(seqLen - len(pwmCdf))])
    motifInstance = ''.join([randomAA(cdf) for cdf in pwmCdf])
    return bgInstance[:st] + motifInstance + bgInstance[st:]

def emitSave(seqFile, pwm, lenMin, lenMax, nSeq, randSeed):
    random.seed(randSeed)
    pwmCdf = [pdf2cdf(pdf) for pdf in pwm]
    f = open(seqFile, 'w')
    for i in xrange(nSeq):
        seq = emit(pwmCdf, lenMin, lenMax)
        f.write('>diag%03d\n%s\n' % (i, seq))
    f.close()

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) > 1 and '-help' == sys.argv[1].lower():
        print USAGE; sys.exit(1)
    pwmLen = 8
    lenMin = 200
    lenMax = 500
    nSeq = 50
    randSeed = 46637
    if len(sys.argv) > 1: pwmLen = int(sys.argv[1])
    if len(sys.argv) > 2: lenMin = int(sys.argv[2])
    if len(sys.argv) > 3: lenMax = int(sys.argv[3])
    if len(sys.argv) > 4: nSeq = int(sys.argv[4])
    if len(sys.argv) > 5: randSeed = int(sys.argv[5])
    if len(sys.argv) > 6: seqFile = sys.argv[6]
    else: seqFile = 'emitdiag%d.fa' % pwmLen

    print 'Generating sequences using parameters (-help for options):'
    print 'PWM length: %d' % pwmLen
    print 'Sequence length: %d ~ %d' % (lenMin, lenMax)
    print 'Number of sequences: %d' % nSeq
    print 'Random seed: %d' % randSeed
    
    diagPwm = createDiagPwm(pwmLen)
    print 'PWM:'
    for pdf in diagPwm:
        print ' '.join(['%02d' % (100 * p) for p in pdf])
    
    emitSave(seqFile, diagPwm, lenMin, lenMax, nSeq, randSeed)
    print 'Sequence generated to FASTA file: %s' % seqFile
