#!/usr/bin/python
#Train one-vs-all SVMs using motif features, either from MEME, gen HMM,
#or disc HMM. One sequence with multi-compartments is supported.
#Assume one feature file, e.g. training data for one fold in cross validation.

import sys
from memecv import sum, fastaReadAll, readFeatures
from feat2acc_fold import COMPS, writeFeatures
from feat2acc_tree import trainSVM

USAGE = 'feat2acc_train FEATFILE FEATMCFILE MODELPAT SEQPAT [SVMOPT]'

def train1(featFile, featMcFile, modelFiles, seqFiles, svmOpt):
    seqId = []; compSize = []
    for seqFilename in seqFiles:
        idSeqs = fastaReadAll(seqFilename)
        seqId.append([idSeq[0].split()[0] for idSeq in idSeqs])
        compSize.append(len(idSeqs))
    feat = readFeatures(compSize, featFile)
    trainSVM(feat, seqId, featMcFile, modelFiles, svmOpt)
    
# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 2: print USAGE; sys.exit(1)
    featFile        = sys.argv[1]
    featMcFile      = sys.argv[2]
    modelPattern    = sys.argv[3]
    seqPattern      = sys.argv[4]
    svmOpt = ''
    if len(sys.argv) > 5: svmOpt = sys.argv[5]
    modelFiles = [modelPattern % c for c in COMPS]
    seqFiles = [seqPattern % c for c in COMPS]
    train1(featFile, featMcFile, modelFiles, seqFiles, svmOpt)
