#!/usr/bin/python
# Train a SVM under a tree topology,
# features including internal and leaf motifs.
# This program supports the deep directory structure and multi-compartments.

import sys
from memecv import fastaReadAll, readFeatures
from feat2acc_tree import nodeList, dfsCompList, groupTree, trainSVM

USAGE = 'feat2acc_treetr FEATFILE FEATMCPAT MODELPAT SEQPAT [SVMOPT] [-b]'

def treeTrain1(trainFeatFile, featMcPattern, modelPattern, trainSeqFiles, binary1Model,\
               svmOpt):
    # filenames (output)
    rootTrFeatMcFile  = featMcPattern % 'YeastRoot'
    secPwTrFeatMcFile = featMcPattern % 'YeastSecPw'
    intraTrFeatMcFile = featMcPattern % 'YeastIntra'
    orgaTrFeatMcFile  = featMcPattern % 'YeastOrga'
    if binary1Model:
        rootModelFiles = [modelPattern % 'YeastSecPw']
        intraModelFiles = [modelPattern % 'YeastOrga']
    else:
        rootModelFiles = [modelPattern % comp for comp in nodeList[:2]]
        intraModelFiles = [modelPattern % comp \
                           for comp in [nodeList[2], nodeList[-1]]]
    secPwModelFiles = [modelPattern % comp for comp in dfsCompList[:5]]
    orgaModelFiles = [modelPattern % comp for comp in dfsCompList[5:8]]

    # initialization    
    # dfsTrFeat/dfsTeFeat: Depth-First-Search Train/Test Features
    trainSeqId = []; trainNSeq = []
    for seqFilename in trainSeqFiles:
        idSeqs = fastaReadAll(seqFilename)
        trainSeqId.append([idSeq[0].split()[0] for idSeq in idSeqs])
        trainNSeq.append(len(idSeqs))
    dfsTrFeat = readFeatures(trainNSeq, trainFeatFile)
    (rootTrFeat, secPwTrFeat, intraTrFeat, orgaTrFeat) = groupTree(dfsTrFeat)
    (rootTrSeqId,secPwTrSeqId,intraTrSeqId,orgaTrSeqId) = groupTree(trainSeqId)

    # train
    trainSVM(rootTrFeat, rootTrSeqId, rootTrFeatMcFile, rootModelFiles, svmOpt)
    trainSVM(secPwTrFeat,secPwTrSeqId,secPwTrFeatMcFile,secPwModelFiles,svmOpt)
    trainSVM(intraTrFeat,intraTrSeqId,intraTrFeatMcFile,intraModelFiles,svmOpt)
    trainSVM(orgaTrFeat, orgaTrSeqId, orgaTrFeatMcFile, orgaModelFiles, svmOpt)

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 5: print USAGE; sys.exit(1)
    trainFeatFile   = sys.argv[1]
    featMcPattern   = sys.argv[2]
    modelPattern    = sys.argv[3]
    seqPattern      = sys.argv[4]
    svmOpt = ''
    if len(sys.argv) > 5: svmOpt = sys.argv[5]
    if len(sys.argv) > 6 and sys.argv[6].lower() == '-b': binary1Model = False
    else: binary1Model = True
    trainSeqFiles = [seqPattern % comp for comp in dfsCompList]
    treeTrain1(trainFeatFile, featMcPattern, modelPattern, trainSeqFiles, \
               binary1Model, svmOpt)
