#!/usr/bin/python
# Run generative motif finding on the Hallett 2004 yeast dataset in K-fold CV,
# under given parameters. The program is SUBMOTIF.
# Run one fold, scanning all compartments.
# Usually run as one job in a server-cluster
# (e.g. qarray "python genhmm_hallett_fold 1 \"-i 100...\"");
# better use genhmm_hallett.py to submit K jobs.
# Assumes the CV training data is ready.
# Results files are stored in current directory

import sys, os, glob

USAGE = 'genhmm_hallett_fold FOLD PARAM [COMPARTMENTS] [MOTIFLEN] [SEED]'

def runComp(param, randSeed, fold, comp, motlen):
    dataFile = '~/subloc/data/hallett2004cv/Yeast%s%str%s.fa' \
               % (comp, randSeed, fold)
    motifFile = 'motif/Yeast%s%str%s.submotif' % (comp, randSeed, fold)
    cmd = '~/subloc/hmmer-2.3.2/subloc/submotif %s -l %d %s %s' \
          % (param, motlen, dataFile, motifFile)
    print cmd
    os.system(cmd)

def runLeafComp(param, randSeed, fold):
    runComp(param, randSeed, fold, 'Cyto',    4)
    runComp(param, randSeed, fold, 'ER',      4) #hdel,rr
    runComp(param, randSeed, fold, 'Golgi',   4) #gol1
    runComp(param, randSeed, fold, 'Mito',    4) #mit1
    runComp(param, randSeed, fold, 'Nuclear', 4) #nuc4, rrm
    runComp(param, randSeed, fold, 'Pero',    3) #pts2
    runComp(param, randSeed, fold, 'PM',      4) #NPxY
    runComp(param, randSeed, fold, 'Lyso',    4) #lys1
    runComp(param, randSeed, fold, 'Secreted',4)

def runComps(comps, motifLen, param, randSeed, fold):
    for comp in comps: runComp(param, randSeed, fold, comp, motifLen)

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 3: print USAGE; sys.exit(1)
    fold = sys.argv[1]
    param = sys.argv[2]
    randSeed = '1000'
    comps = None
    motifLen = 4
    if len(sys.argv) > 3: comps = sys.argv[3].split(',')
    if len(sys.argv) > 4: motifLen = int(sys.argv[4])
    if len(sys.argv) > 5: randSeed = sys.argv[5]
    if comps == None: runLeafComp(param, randSeed, fold)
    else: runComps(comps, motifLen, param, randSeed, fold)
