#Summarize the known motifs found by a program (MEME, gen-hmm, disc-hmm)
#Q: Total number of known motifs?
#A: Either an argument, or from *.motpos, or record in *.motres

import sys

USAGE = 'hit2summary NUM-KNOWN-MOTIF RESULT-PATTERN PR-FILE [FOLDS]'

def summarizeHits(resultFiles, prFile, nTrue):
    nPred = 10; nComp = 9; nFold = len(resultFiles)
    hitAllFolds = 0
    foundDict = {}
    tp = [0] * nPred
    for resultFile in resultFiles:
        f = open(resultFile)
        hasNMotifLine = False
        for line in f:
            if line.startswith('Yeast'): #Motif result line
                tokens = line.split()
                motifName = tokens[1]; foundDict[motifName] = 1
                rank = int(tokens[7])
                for rank1 in range(rank-1, nPred):
                    tp[rank1] = tp[rank1] + 1
            elif line.find('KNOWN MOTIFS FOUND') >= 0:
                hitAllFolds = hitAllFolds + int(line.split()[0])
                hasNMotifLine = True
        f.close()
        assert hasNMotifLine
    # report1: average hits
    hitAvg = float(hitAllFolds) / nFold
    print '%.1f known motifs found, averaged over %d folds' % (hitAvg, nFold)
    # report2: unique motifs found
    uniqueMotifs = foundDict.keys()
    print '%d unique motifs:' % len(uniqueMotifs)
    for motif in uniqueMotifs: print motif
    # report3: precision-recall by rank
    savePrecRecByRank(prFile, tp, nTrue, nComp, nFold)

def savePrecRecByRank(prFile, tp, nTrue, nComp, nFold):
    f = open(prFile, 'w')
    for rank in range(len(tp)):
        prec = float(tp[rank]) / nComp / (rank+1) / nFold
        rec = float(tp[rank]) / nTrue / nFold
        f.write('%.3f\t%.3f\n' % (prec, rec))
    f.close()

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 4: print USAGE; sys.exit(1)
    nTrue = int(sys.argv[1])
    resultPattern = sys.argv[2]
    prFile = sys.argv[3]
    nFold = 10
    if len(sys.argv) > 4: nFold = int(sys.argv[4])
    if '%' not in resultPattern: nFold = 1
    if nFold > 1:
        resultFiles = [resultPattern % fold for fold in range(nFold)]
    else:
        resultFiles = [resultPattern]
    summarizeHits(resultFiles, prFile, nTrue)
