#!/usr/bin/python

import sys, os
from memecv import writeFeatures

USAGE = 'hmm2feat_fold FOLD [MOTIFLEN] [HMMDIR] [SEED]'

COMPS_LEAF=['Cyto','ER','Golgi','Mito','Nuclear','Pero','PM','Lyso','Secreted']
DEFAULT_MOTIF_LEN = [4, 4, 4, 4, 4, 3, 4, 4, 4]

def runComps(randSeed, fold, motifLens, hmmDir, comps):
    hmmFiles = ['%sYeast%s%str%s%s_' % (hmmDir,comp,randSeed,fold,len) \
                for comp, len in zip(comps, motifLens)]
    for comp in comps: runComp(randSeed, fold, comp, ' '.join(hmmFiles))

    print 'Combining compartment features...'
    combineFold(randSeed, fold, comps)    

def runComp(randSeed, fold, comp, hmmFileList):
    dataFile = '~/subloc/data/hallett2004cv/Yeast%s%str%s.fa' \
               % (comp, randSeed, fold)
    featFile = 'features_comp/Yeast%s%str%s.dat' % (comp, randSeed, fold)
    cmd = '~/subloc/hmmer-2.3.2/subloc/subfeat -v 1 %s %s %s' \
          % (dataFile, featFile, hmmFileList)
    print cmd; os.system(cmd)

    dataFile = '~/subloc/data/hallett2004cv/Yeast%s%ste%s.fa' \
               % (comp, randSeed, fold)
    featFile = 'features_comp/Yeast%s%ste%s.dat' % (comp, randSeed, fold)
    cmd = '~/subloc/hmmer-2.3.2/subloc/subfeat -v 1 %s %s %s' \
          % (dataFile, featFile, hmmFileList)
    print cmd; os.system(cmd)

def combineFold(seed, fold, comps):
    trainFeat = []; testFeat = []
    for comp in comps:
        trainFeat.append(readArray('features_comp/Yeast%s%dtr%d.dat' \
                                   % (comp, seed, fold)))
        testFeat.append( readArray('features_comp/Yeast%s%dte%d.dat' \
                                   % (comp, seed, fold)))
    trainFeatFile = 'features/Yeast%dtr%d.dat'   % (seed, fold)
    testFeatFile  = 'features/Yeast%dte%d.dat' % (seed, fold)
    writeFeatures(trainFeat, -1, trainFeatFile)
    writeFeatures(testFeat,  -1, testFeatFile)

def readArray(featFilename):
    fFeat = open(featFilename)
    featVecs = []
    for line in fFeat:
        if line == '': continue
        featVecs.append([float(token) for token in line.split()])
    fFeat.close()
    return featVecs

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 2: print USAGE; sys.exit(1)
    fold = sys.argv[1];
    motifLens = DEFAULT_MOTIF_LEN; hmmDir = 'hmm/'; comps = COMPS_LEAF;
    randSeed = '1000'
    if len(sys.argv) > 2: motifLens = [int(sys.argv[2])] * len(comps)
    if len(sys.argv) > 3: hmmDir = sys.argv[3]
    if len(sys.argv) > 4: randSeed = sys.argv[4]
    runComps(randSeed, fold, motifLens, hmmDir, comps)
