import sys, os
from memecv import writeFeatures
from hmm2feat_fold import readArray

USAGE = 'hmm2feat_tree FOLD [BRANCH_MOTIF_LEN] [SEED]'

nodeList = ['YeastSecPw', 'YeastIntra', 'YeastOrga',
            'YeastER', 'YeastGolgi', 'YeastLyso', 'YeastPM', 'YeastSecreted',
            'YeastCyto', 'YeastPero', 'YeastMito', 'YeastNuclear']
dfsCompList = nodeList[3:]
branchList = nodeList[:3]

DEFAULT_MOTIF_LEN = [4, 4, 4, 4, 4, 4, 3, 4, 4]

def subfeat(seqFile, featFile, hmmFileList):
    cmd = '~/subloc/hmmer-2.3.2/subloc/subfeat %s %s %s' \
          % (seqFile, featFile, hmmFileList)
    print cmd; os.system(cmd)

def hmm2featTree(seed, fold, brMotifLens):
    # Alternative way to avoid re-calculating features:
    # for each comp, run subfeat with branches
    # for each comp, readArray branch, readArray leaf
    # combine branch and leaf features, write combined features

    motifLens = brMotifLens + DEFAULT_MOTIF_LEN
    hmmList = ' '.join(['hmm/%s%str%s%d_' % (comp,seed,fold,len) \
                        for comp,len in zip(nodeList, motifLens)])
    trainFeat = []; testFeat = []
    for comp in dfsCompList:
        # filenames
        trSeqFile = '~/subloc/data/hallett2004cv/%s%str%s.fa' % (comp,seed,fold)
        teSeqFile = '~/subloc/data/hallett2004cv/%s%ste%s.fa' % (comp,seed,fold)
        trFeatCompFile = 'features_comp/Tree%s%str%s.dat'     % (comp,seed,fold)
        teFeatCompFile = 'features_comp/Tree%s%ste%s.dat'     % (comp,seed,fold)
        # call subfeat
        subfeat(trSeqFile, trFeatCompFile, hmmList)
        subfeat(teSeqFile, teFeatCompFile, hmmList)
        # read and combine compartment array features
        trainFeat.append(readArray(trFeatCompFile))
        testFeat.append( readArray(teFeatCompFile))
    trainFeatFile = 'features/YeastTree%str%s.dat'   % (seed, fold)
    testFeatFile  = 'features/YeastTree%ste%s.dat' % (seed, fold)
    writeFeatures(trainFeat, -1, trainFeatFile)
    writeFeatures(testFeat,  -1, testFeatFile)

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 2: print USAGE; sys.exit(1)
    fold = sys.argv[1];
    brMotifLens = [4] * 3; seed = '1000'
    if len(sys.argv) > 2: brMotifLens = [int(sys.argv[2])] * 3
    if len(sys.argv) > 3: seed = sys.argv[3]
    hmm2featTree(seed, fold, brMotifLens)
