#!/usr/bin/python
#This script overlaps and replaces hmm2feat_fold.py

import sys, os
from memecv import writeFeatures

USAGE = '''hmm2flatfeat TR_FEAT TE_FEAT HMM TR_SEQ TE_SEQ
All parameters are printf-style patterns (e.g. hmm/%s1000tr3.hmm).'''

COMPS = ['YeastCyto', 'YeastER', 'YeastGolgi', 'YeastLyso', 'YeastMito',
         'YeastNuclear', 'YeastPero', 'YeastPM', 'YeastSecreted']

def runComps(comps, trSeqPattern, teSeqPattern, trFeatPattern, teFeatPattern, \
             hmmPattern):
    hmmFiles = [hmmPattern % comp for comp in comps]
    hmmFileList = ' '.join(hmmFiles)
    for comp in comps:
        trSeqFile = trSeqPattern % comp
        trFeatFile = trFeatPattern % comp
        cmd = '~/subloc/hmmer-2.3.2/subloc/subfeat -v 1 %s %s %s' \
              % (trSeqFile, trFeatFile, hmmFileList)
        print cmd; os.system(cmd)

        teSeqFile = teSeqPattern % comp
        teFeatFile = teFeatPattern % comp
        cmd = '~/subloc/hmmer-2.3.2/subloc/subfeat -v 1 %s %s %s' \
              % (teSeqFile, teFeatFile, hmmFileList)
        print cmd; os.system(cmd)

    print 'Combining compartment features...'
    combineFold(comps, trFeatPattern, teFeatPattern)

def combineFold(comps, trFeatPattern, teFeatPattern):
    trainFeat = []; testFeat = []
    for comp in comps:
        trainFeat.append(readArray(trFeatPattern % comp))
        testFeat.append( readArray(teFeatPattern % comp))
    trainFeatFile = trFeatPattern % 'Yeast'
    testFeatFile  = teFeatPattern % 'Yeast'
    writeFeatures(trainFeat, -1, trainFeatFile)
    writeFeatures(testFeat,  -1, testFeatFile)

def readArray(featFilename):
    fFeat = open(featFilename)
    featVecs = []
    for line in fFeat:
        if line == '': continue
        featVecs.append([float(token) for token in line.split()])
    fFeat.close()
    return featVecs

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 6: print USAGE; sys.exit(1)
    trFeatPattern = sys.argv[1]
    teFeatPattern = sys.argv[2]
    hmmPattern    = sys.argv[3]
    trSeqPattern  = sys.argv[4]
    teSeqPattern  = sys.argv[5]
    #if hmmPattern.find('%s') < 0: hmmPattern = hmmPattern + '%s.hmm'
    runComps(COMPS, trSeqPattern, teSeqPattern, trFeatPattern, teFeatPattern, \
             hmmPattern)
