#!/usr/bin/python

import sys, os
from memecv import writeFeatures

USAGE = '''hmm2flatfeat TE_FEAT HMM TE_SEQ
All parameters are printf-style patterns (e.g. hmm/%s1000tr3.hmm).'''

COMPS = ['YeastCyto', 'YeastER', 'YeastGolgi', 'YeastLyso', 'YeastMito',
         'YeastNuclear', 'YeastPero', 'YeastPM', 'YeastSecreted']

def runComps(comps, teSeqPattern, teFeatPattern, hmmPattern):
    hmmFiles = [hmmPattern % comp for comp in comps]
    hmmFileList = ' '.join(hmmFiles)
    for comp in comps:
        teSeqFile = teSeqPattern % comp
        teFeatFile = teFeatPattern % comp
        cmd = '~/subloc/hmmer-2.3.2/subloc/subfeat -v 1 %s %s %s' \
              % (teSeqFile, teFeatFile, hmmFileList)
        print cmd; os.system(cmd)

    print 'Combining compartment features...'
    combineFold(comps, teFeatPattern)

def combineFold(comps, teFeatPattern):
    testFeat = []
    for comp in comps:
        testFeat.append( readArray(teFeatPattern % comp))
    testFeatFile  = teFeatPattern % 'Yeast'
    writeFeatures(testFeat,  -1, testFeatFile)

def readArray(featFilename):
    fFeat = open(featFilename)
    featVecs = []
    for line in fFeat:
        if line == '': continue
        featVecs.append([float(token) for token in line.split()])
    fFeat.close()
    return featVecs

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 4: print USAGE; sys.exit(1)
    teFeatPattern = sys.argv[1]
    hmmPattern    = sys.argv[2]
    teSeqPattern  = sys.argv[3]
    #if hmmPattern.find('%s') < 0: hmmPattern = hmmPattern + '%s.hmm'
    runComps(COMPS, teSeqPattern, teFeatPattern, hmmPattern)
