#!/usr/bin/python
#Generate features of HMM likelihood for flat, leaf, or tree SVM classification.
#This script overlaps and replaces both hmm2feat_tree.py (supporting hmmd
#with printf-style patterns) and hmm2featflat.py (supporting tree and flat).
#To support different directory structure of hmmd, a more flexible
#command-line arguments is added.
#To support tree and flat classification, HMM compartment list (tree or leaf)
#and sequence compartment list are separated.

import sys, os
from memecv import writeFeatures
from hmm2featflat import readArray

USAGE = '''Usage: hmm2feattree TR_FEAT TE_FEAT HMM TR_SEQ TE_SEQ [-t]
Arguments:
  TR_FEAT output training feature file pattern (ex. feattree/Yeast1000tr3.dat)
  TE_FEAT output testing feature file pattern (ex. feattree/Yeast1000te3.dat)
  HMM     HMM file pattern (ex. hmmd/%s1000tr3.hmm)
  TR_SEQ  training sequences
  TE_SEQ  testing sequences
  -t      tree features
'''

nodeList = ['YeastSecPw', 'YeastIntra', 'YeastOrga',
            'YeastER', 'YeastGolgi', 'YeastLyso', 'YeastPM', 'YeastSecreted',
            'YeastCyto', 'YeastPero', 'YeastMito', 'YeastNuclear']
leafList = nodeList[3:]

def runComps(comps, trSeqPattern, teSeqPattern, trFeatPattern, teFeatPattern, \
             hmmFileList):
    for comp in comps:
        trSeqFile = trSeqPattern % comp
        trFeatFile = trFeatPattern % comp
        cmd = '~/subloc/hmmer-2.3.2/subloc/subfeat -v 1 %s %s %s' \
              % (trSeqFile, trFeatFile, hmmFileList)
        print cmd; os.system(cmd)

        teSeqFile = teSeqPattern % comp
        teFeatFile = teFeatPattern % comp
        cmd = '~/subloc/hmmer-2.3.2/subloc/subfeat -v 1 %s %s %s' \
              % (teSeqFile, teFeatFile, hmmFileList)
        print cmd; os.system(cmd)

    print 'Combining compartment features...'
    combineFold(comps, trFeatPattern, teFeatPattern)

def combineFold(comps, trFeatPattern, teFeatPattern):
    trainFeat = []; testFeat = []
    for comp in comps:
        trainFeat.append(readArray(trFeatPattern % comp))
        testFeat.append( readArray(teFeatPattern % comp))
    trainFeatFile = trFeatPattern % 'Yeast'
    testFeatFile  = teFeatPattern % 'Yeast'
    writeFeatures(trainFeat, -1, trainFeatFile)
    writeFeatures(testFeat,  -1, testFeatFile)

def readArray(featFilename):
    fFeat = open(featFilename)
    featVecs = []
    for line in fFeat:
        if line == '': continue
        featVecs.append([float(token) for token in line.split()])
    fFeat.close()
    return featVecs

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 6: print USAGE; sys.exit(1)
    trFeatPattern = sys.argv[1]
    teFeatPattern = sys.argv[2]
    hmmPattern    = sys.argv[3]
    trSeqPattern  = sys.argv[4]
    teSeqPattern  = sys.argv[5]
    isTree        = False
    if len(sys.argv) > 6:
        if sys.argv[6].lower() == '-t': isTree = True

    if not isTree: hmmCompList = leafList
    else:          hmmCompList = nodeList
    hmmFileList = ' '.join([hmmPattern % comp for comp in hmmCompList])
    runComps(nodeList, trSeqPattern, teSeqPattern, trFeatPattern, teFeatPattern, \
             hmmFileList)
