#Read InterProScan results, filter based on relevance to compartments,
#and write relevant motif instances to the position files.
#Use default parameters and no need to test several parameters hopefully

LEN_TH = 40
COUNT_TH = 4
EMPTY_COMP = 6

#This will never be displayed
USAGE = '''ipr2motif.py [POS_PATTERN] [COUNT_FILE] [IPRSCAN_PATTERN]
ex. ipr2motif.py
ex. ipr2motif.py %s.iprpos ipr.count %s.iprscan'''

import sys
from scanipr import COMPS, writeMotPos, printMotifCounts

def selectIpr(iprFiles, posFiles, countFile):
    compIprPos = []; motifList = []
    for iprFile in iprFiles:
        compIprPos.append(countCompIpr(iprFile, motifList))
    fillEmptyMotif(compIprPos, motifList)
    writeCounts(compIprPos, countFile)
    selCompIprPos = selectIprByCount(compIprPos)
    writeSelPositions(selCompIprPos, motifList, posFiles, iprFiles)

def countCompIpr(iprFile, motifList):
    iprPos = [[] for motif in motifList]
    f = open(iprFile)
    for line in f:
        #assuming number of columns (seperated by tabs) is always correct
        (seqId, crc, seqLen, db, dbId, dbDesc, start, end, eVal, status, \
         date, iprName, go) = line.split('\t')
        if iprName in motifList: iMotif = motifList.index(iprName)
        else: motifList.append(iprName); iprPos.append([]); iMotif = -1
        #0 inserted as seq-number to be compatible with scanregex
        iprPos[iMotif].append((seqId, 0, int(start), int(end)+1))
    f.close()
    return zip(motifList, iprPos)

def fillEmptyMotif(compIprPos, motifList):
    nMotif = len(compIprPos[-1])
    for iprPos in compIprPos[:-1]:
        for i in range(len(iprPos), nMotif): iprPos.append((motifList[i], []))
        assert len(iprPos) == nMotif

def writeCounts(compIprPos, countFile):
    f = open(countFile, 'w')
    for iprPos in compIprPos:
        f.write('\t'.join(['%d' % len(pos) for mot, pos in iprPos]) + '\n')
    f.close()

def selectIprByCount(compIprPos):
    selCompIprPos = [[] for iprPos in compIprPos]
    for iMotif in range(len(compIprPos[0])):
        isAbove = False; n0 = 0
        for iprPos in compIprPos:
            count = len(iprPos[iMotif][1])
            if count >= COUNT_TH: isAbove = True
            elif count == 0: n0 = n0 + 1
        if n0 >= EMPTY_COMP and isAbove:
            for selIprPos, iprPos in zip(selCompIprPos, compIprPos):
                selIprPos.append(iprPos[iMotif])
    return selCompIprPos

def writeSelPositions(selCompIprPos, motifList, posFiles, iprFiles):
    nMotif = 0
    for iprPos, posFile, iprFile in zip(selCompIprPos, posFiles, iprFiles):
        writeMotPos(posFile, iprPos)
        nonEmpty = len([1 for motif, posList in iprPos if len(posList)>0])
        nMotif = nMotif + nonEmpty
        print '%s: %d motifs' % (posFile, nonEmpty)
    print 'Sum of motif number: %d' % nMotif
    print '%d unique motifs written.' % len(selCompIprPos[-1])

# main
if __name__ == '__main__': #allows import from interpreter
    iprPattern='%s.iprscan'; posPattern='%s.iprpos'; countFile='ipr.count'
    #if len(sys.argv) < 2: print USAGE; sys.exit(1)
    if len(sys.argv) > 1: posPattern = sys.argv[1]
    if len(sys.argv) > 2: countFile = sys.argv[2]
    if len(sys.argv) > 3: iprPattern = sys.argv[3]
    iprFiles = [iprPattern % comp for comp in COMPS]
    posFiles = [posPattern % comp for comp in COMPS]
    selectIpr(iprFiles, posFiles, countFile)
