#This script overlaps memecv.py (memecv.py feat ...) but supports
#the deep directory structure and more flexible file names.
#The usage is similar to hmm2featflat.py.

import sys, os
from memecv import memeFeatures, writeFeatures

USAGE = '''meme2flatfeat TR_FEAT TE_FEAT MEME TR_SEQ TE_SEQ
All parameters are printf-style patterns (e.g. motif/%s1000tr3.meme).
See script files for examples.'''

COMPS = ['YeastCyto', 'YeastER', 'YeastGolgi', 'YeastLyso', 'YeastMito',
         'YeastNuclear', 'YeastPero', 'YeastPM', 'YeastSecreted']

def runComps(comps, memePattern, trSeqPattern, teSeqPattern, trFeatFile, teFeatFile):
    '''Generate features from motif PWM.'''
    trSeqFiles = [trSeqPattern % comp for comp in comps]
    teSeqFiles = [teSeqPattern % comp for comp in comps]
    memeFiles = [memePattern % comp for comp in comps]
    (trainFeat, testFeat) = memeFeatures(memeFiles, trSeqFiles, teSeqFiles)
    #featNormalize(trainFeat, testFeat)
    writeFeatures(testFeat , -1, teFeatFile)
    writeFeatures(trainFeat, -1, trFeatFile)

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 6: print USAGE; sys.exit(1)
    trFeatFile   = sys.argv[1]
    teFeatFile   = sys.argv[2]
    memePattern  = sys.argv[3]
    trSeqPattern = sys.argv[4]
    teSeqPattern = sys.argv[5]
    runComps(COMPS, memePattern, trSeqPattern, teSeqPattern, trFeatFile, teFeatFile)
