#This script overlaps memecvtree.py but supports
#the deep directory structure and more flexible file names.
#The usage is similar to hmm2feattree.py.

import sys, os
from memecv import memeFeatures, writeFeatures

USAGE = '''meme2flattree TR_FEAT TE_FEAT MEME TR_SEQ TE_SEQ [-t]
Arguments:
  TR_FEAT output training feature file pattern (ex. feattree/Yeast1000tr3.dat)
  TE_FEAT output testing feature file pattern (ex. feattree/Yeast1000te3.dat)
  MEME    MEME file pattern (ex. motif/%s1000tr3.meme)
  TR_SEQ  training sequences
  TE_SEQ  testing sequences
  -t      tree features
All parameters are printf-style patterns (e.g. motif/%s1000tr3.meme).
See script files for examples.'''

NODE_LIST = ['YeastSecPw', 'YeastIntra', 'YeastOrga',
             'YeastER', 'YeastGolgi', 'YeastLyso', 'YeastPM', 'YeastSecreted',
             'YeastCyto', 'YeastPero', 'YeastMito', 'YeastNuclear']
LEAF_LIST = NODE_LIST[3:]

def runComps(comps, memeComps, memePattern, trSeqPattern, teSeqPattern, \
             trFeatFile, teFeatFile):
    '''Generate features from motif PWM.'''
    trSeqFiles = [trSeqPattern % comp for comp in comps]
    teSeqFiles = [teSeqPattern % comp for comp in comps]
    memeFiles = [memePattern % comp for comp in memeComps]
    (trainFeat, testFeat) = memeFeatures(memeFiles, trSeqFiles, teSeqFiles)
    #featNormalize(trainFeat, testFeat)
    writeFeatures(testFeat , -1, teFeatFile)
    writeFeatures(trainFeat, -1, trFeatFile)

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 6: print USAGE; sys.exit(1)
    trFeatFile   = sys.argv[1]
    teFeatFile   = sys.argv[2]
    memePattern  = sys.argv[3]
    trSeqPattern = sys.argv[4]
    teSeqPattern = sys.argv[5]
    isTree        = False
    if len(sys.argv) > 6:
        if sys.argv[6].lower() == '-t': isTree = True

    if not isTree: memeComps = LEAF_LIST
    else:          memeComps = NODE_LIST
    runComps(NODE_LIST, memeComps, memePattern, trSeqPattern, teSeqPattern, \
             trFeatFile, teFeatFile)
