#This script overlaps memecvtree.py but supports
#the deep directory structure and more flexible file names.
#The usage is similar to hmm2feattree.py.

import sys, os
from memecv import memeFeatures, writeFeatures

USAGE = '''meme2flattree TE_FEAT MEME TE_SEQ [-t]
All parameters are printf-style patterns (e.g. motif/%s1000tr3.meme).
See script files for examples.'''

NODE_LIST = ['YeastSecPw', 'YeastIntra', 'YeastOrga',
             'YeastER', 'YeastGolgi', 'YeastLyso', 'YeastPM', 'YeastSecreted',
             'YeastCyto', 'YeastPero', 'YeastMito', 'YeastNuclear']
LEAF_LIST = NODE_LIST[3:]

def runComps(comps, memeComps, memePattern, teSeqPattern, teFeatFile):
    '''Generate features from motif PWM.'''
    teSeqFiles = [teSeqPattern % comp for comp in comps]
    memeFiles = [memePattern % comp for comp in memeComps]
    (trainFeat, testFeat) = memeFeatures(memeFiles, [], teSeqFiles)
    #featNormalize(trainFeat, testFeat)
    writeFeatures(testFeat, -1, teFeatFile)

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) < 4: print USAGE; sys.exit(1)
    teFeatFile   = sys.argv[1]
    memePattern  = sys.argv[2]
    teSeqPattern = sys.argv[3]
    isTree        = False
    if len(sys.argv) > 4:
        if sys.argv[4].lower() == '-t': isTree = True

    if not isTree: memeComps = LEAF_LIST
    else:          memeComps = NODE_LIST
    runComps(NODE_LIST, memeComps, memePattern, teSeqPattern, teFeatFile)
