import sys, getopt, time
from memecv import readMeme, fastaReadAll, pwmLogLik
from motif2hit import COMPS, readMotPos, writeResultWithRank
from posterior2hit import paramPredToTrue, paramRecallThreshold, paramOverlapRatio, postToHit

USAGE = '''memesort2hit.py [-options] RESULT_FILE MEME_PAT MOTPOS_PAT SEQ_PAT
Evaluate predicted motif instances.

MEME_PAT and MOTPOS_PAT use the %s symbol,
e.g. ~/subloc/result/hallett2005/%s.iprpos.

Options:
   -p <Pred-to-true>    : prediction-to-true ratio
   -o <Overlap>         : overlap threshold
   -r <Recall>          : recall threshold        '''

def allPostToHit(motPosFiles, memeFiles, resultFile, seqFiles, comps):
    compResults = []
    for motPosFile, memeFile, seqFile in zip(motPosFiles, memeFiles, seqFiles):
        print 'Matching %s...' % memeFile
        motNameInstances = readMotPos(motPosFile)
        if len(motNameInstances) == 0: compResults.append([]); continue
        sortedMotPos = readSortedMeme(memeFile, seqFile)
        compResults.append(postToHit(motNameInstances, sortedMotPos))
    title = 'Pred/true=%d,Recall>=%g,Overlap>=%g' \
            % (paramPredToTrue, paramRecallThreshold, paramOverlapRatio)
    writeResultWithRank(title, compResults, comps, resultFile)

def readSortedMeme(memeFile, seqFile):
    pwms = readMeme(memeFile)
    nameSeqs = fastaReadAll(seqFile)

    sortedMotPos = []
    for pwm in pwms:
        w = len(pwm)
        posList = []
        for name, seq in nameSeqs:
            pll = [(pwmLogLik(pwm, seq[p:p+w]), p, seq[p:p+w]) \
                   for p in range(len(seq)-w+1)]
            pll.sort(); pll.reverse(); pll = pll[:10]
            posList = posList + [(ll,name,p,p+w,sites) for ll,p,sites in pll]
        posList.sort(); posList.reverse(); posList = posList[:2000]
        sortedMotPos.append([x[1:]+x[:1] for x in posList])
    return sortedMotPos


# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) == 3: #TEST!
        t0 = time.time()
        sortedMotPos = readSortedMeme(sys.argv[1], sys.argv[2])
        for iMotif in range(len(sortedMotPos)):
            print 'Motif %d' % iMotif
            for pos in sortedMotPos[iMotif][:10]:
                print pos[2]-pos[1], pos
            print
        print 'Elapsed time: %.0f sec' % (time.time() - t0)
        sys.exit();

    try:
        (opts, args) = getopt.getopt(sys.argv[1:], 'p:o:r:')
    except getopt.GetoptError:
        print USAGE; sys.exit(2)
    for o, a in opts:
        if   o == '-p': paramPredToTrue      = int(a)
        elif o == '-o': paramOverlapRatio    = float(a)
        elif o == '-r': paramRecallThreshold = float(a)
    if len(args) != 4: print USAGE; sys.exit(2)
    (resultFile, memeFilePattern, motPosFilePattern, seqPattern) = args

    t0 = time.time()
    memeFiles = [memeFilePattern % comp for comp in COMPS]
    motPosFiles = [motPosFilePattern % comp for comp in COMPS]
    seqFiles = [seqPattern % comp for comp in COMPS]
    allPostToHit(motPosFiles, memeFiles, resultFile, seqFiles, COMPS)
    print 'Elapsed time: %.0f sec' % (time.time() - t0)
