'''rank2postsites.py - select motifs and conserved sites (proteins)
Related to motif2hit.py and motifrank2alignhit.py
'''

import sys
from motif2hit import COMPS
from memesort2hit import readSortedMeme
from motifrank2alignhit import ALIGNMENT_HEAD, ALIGNMENT_BOTTOM, readRank

USAGE = \
'''rank2postsites NSITE ALIGNFILE RANKFILE POSTPAT SEQPAT
Example:
rank2postsites.py $2 $1/Yeast.post${2}sites $1/Yeast.result postflat/%s.post ~/subloc/result/hallett2005/%s.fa
'''

def rank2postsites(rankFile, alignFile, memeFiles, seqFiles, nSite, comps):
    #load and filter posterior-motif-sites
    compMotifSites = []
    for memeFile, seqFile in zip(memeFiles, seqFiles):
        print 'Processing %s...' % memeFile
        sortedMotPos = readSortedMeme(memeFile, seqFile)        
        for postMotif in sortedMotPos: del postMotif[nSite:]
        compMotifSites.append(sortedMotPos)
    #save alignment
    print 'Write alignment to %s...' % alignFile
    writeAnnoRankAlign(rankFile, compMotifSites, alignFile, comps)

def writeAnnoRankAlign(rankFile, compMotifSites, alignFile, comps):
    'Write annotated ranking file and the conserved alignment file.'
    motifRank = readRank(rankFile, comps)
    alignFp = open(alignFile, 'w')
    for i, iComp, comp, order in motifRank:
        writeAlignment(alignFp, compMotifSites, iComp, comp, order, i)
    alignFp.close()

def writeAlignment(alignFp, compMotifSites, iComp, comp, order, rank):
    '''Filter conserved instances and write to the file.
    Note that order is the order of initial finding of a motif;
    rank is the ranking of discriminative power.
    The list of sites in compMotifSites[iComp][order-1] must be filtered;
    the site string in compMotifSites[iComp][order-1][ins][3] must be ready.'''
    motifSite = compMotifSites[iComp][order - 1]
    alignFp.write(ALIGNMENT_HEAD % (rank, comp + str(order), ''))
    for seqId, st, end, site, pp in motifSite:
        alignFp.write('%-25s %5d %9.4f            %s\n' % (seqId,st+1,pp,site))
    alignFp.write(ALIGNMENT_BOTTOM)

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) != 6: print USAGE; sys.exit(1)
    (nSiteStr, alignFile, rankFile, memePattern, seqPattern) = sys.argv[1:]
    nSite = int(nSiteStr)
    memeFiles = [memePattern % c for c in COMPS]
    seqFiles = [seqPattern % c for c in COMPS]
    rank2postsites(rankFile, alignFile, memeFiles, seqFiles, nSite, COMPS)
