'''rank2postsites.py - select motifs and conserved sites (proteins)
Related to motif2hit.py and motifrank2alignhit.py
'''

import sys
from motif2hit import COMPS
from memecv import fastaReadAll
from posterior2hit import readPosterior
from motifrank2alignhit import ALIGNMENT_HEAD, ALIGNMENT_BOTTOM, readRank

MAX_SITE_LEN = 12
MIN_SITE_LEN = 3

USAGE = \
'''rank2postsites NSITE ALIGNFILE RANKFILE POSTPAT SEQPAT
Example:
rank2postsites.py $2 $1/Yeast.post${2}sites $1/Yeast.result postflat/%s.post ~/subloc/result/hallett2005/%s.fa
'''

def rank2postsites(rankFile, alignFile, postFiles, seqFiles, nSite, comps):
    #load, filter, and look up posterior-motif-sites
    compMotifSites = []
    for postFile, seqFile in zip(postFiles, seqFiles):
        print 'Processing %s...' % postFile,
        nameSeqs = fastaReadAll(seqFile)
        name2seq = dict(nameSeqs)
        sortedMotPos = readPosterior(postFile)
        print
        filteredMotifs = []
        for postMotif in sortedMotPos:
            filteredMotif = []
            for iIns in range(len(postMotif)):
                (seqId, st, end, site, pp) = postMotif[iIns]
                #filter: site length
                if end - st > MAX_SITE_LEN or end - st < MIN_SITE_LEN: continue
                #look up: fill in sites from sequence
                filteredMotif.append( (seqId, st, end, name2seq[seqId][st:end], pp) )
                #select: top K sites
                if len(filteredMotif) >= max(nSite): break
            filteredMotifs.append(filteredMotif)
        compMotifSites.append(filteredMotifs)
    #save alignment
    print 'Write alignment to %s...' % alignFile
    writeAnnoRankAlign(rankFile, compMotifSites, alignFile, comps)

def writeAnnoRankAlign(rankFile, compMotifSites, alignFile, comps):
    'Write annotated ranking file and the conserved alignment file.'
    motifRank = readRank(rankFile, comps)
    for nSite1 in nSite:
        alignFp = open(alignFile % nSite1, 'w')
        j = 0
        for i, iComp, comp, order in motifRank:
            motifSite = compMotifSites[iComp][order - 1]
            if len(motifSite) < nSite1: continue # filter motifs
            j = j + 1
            writeAlignment(alignFp, motifSite, comp, order, j, nSite1)
        alignFp.close()

def writeAlignment(alignFp, motifSite, comp, order, rank, nSite1):
    '''Filter conserved instances and write to the file.
    Note that order is the order of initial finding of a motif;
    rank is the ranking of discriminative power.
    The list of sites in compMotifSites[iComp][order-1] must be filtered;
    the site string in compMotifSites[iComp][order-1][ins][3] must be ready.'''
    alignFp.write(ALIGNMENT_HEAD % (rank, comp + str(order), ''))
    for seqId, st, end, site, pp in motifSite[:nSite1]:
        alignFp.write('%-25s %5d %9.4f            %s\n' % (seqId,st+1,pp,site))
    alignFp.write(ALIGNMENT_BOTTOM)

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) != 6: print USAGE; sys.exit(1)
    (nSiteStr, alignFile, rankFile, postPattern, seqPattern) = sys.argv[1:]

    comps = COMPS # tree would be different

    nSite = [int(s) for s in nSiteStr.split(',')]
    postFiles = [postPattern % c for c in comps]
    seqFiles = [seqPattern % c for c in comps]
    rank2postsites(rankFile, alignFile, postFiles, seqFiles, nSite, comps)
