#!/usr/bin/python
#Filter interproscan results based on pre-selected motifs, e.g. from PSLT2
#This script is similar to scanregex. Most differences are explained in remarks.

import re, time, sys, glob

USAGE = '''scanipr.py MOTIF-FILE [POS-PATTERN] [IPRSCAN-PATTERN]
ex. scanipr.py pslt90.motif'''

COMPS = ['YeastCyto', 'YeastER', 'YeastGolgi', 'YeastLyso', 'YeastMito',
         'YeastNuclear', 'YeastPero', 'YeastPM', 'YeastSecreted']

def scanWriteAllIpr(iprFiles, posFiles, motifFile):
    '''Scan all sequence files with all corresponding motifs,
    and write to corresponding position files'''
    motifLists = readMotifLists(motifFile)
    totalNMotif = 0
    for iprFile, posFile, motifList in zip(iprFiles, posFiles, motifLists):
        motifPos = scanInterPro(iprFile, motifList)
        writeMotPos(posFile, motifPos)
        printMotifCounts(iprFile, motifPos)
        totalNMotif = totalNMotif + len(motifPos)
    print 'Total number of motif:', totalNMotif

def readMotifLists(motifFile):
    motifLists = [None] * len(COMPS)
    f = open(motifFile)
    for line in f:
        tokens = line.split()
        compName = tokens[0]
        motifLists[COMPS.index(compName)] = tokens[1:]
    f.close()
    return motifLists

def scanInterPro(iprFile, motifList):
    posByMotif = [[] for motif in motifList]
    f = open(iprFile)
    for line in f:
        #assuming number of columns (seperated by tabs) is always correct
        (seqId, crc, seqLen, db, dbId, dbDesc, start, end, eVal, status, \
         date, iprName, go) = line.split('\t')
        if iprName in motifList:
            i = motifList.index(iprName)
            #0 inserted as seq-number to be compatible with scanregex
            posByMotif[i].append((seqId, 0, int(start), int(end)+1))
    f.close()
    #Return filtered list such that there are more than 5 occurrences.
    #Note that a list of tuples of motif and position-list is returned,
    #instead of just the position-list in scanregex.
    return [(motif, posList) for motif, posList in zip(motifList, posByMotif) \
            if len(posList) >= 5]

def writeMotPos(posFile, motifPos):
    fPos = open(posFile, 'w')
    fPos.write('#Motif\tRegex\tSeq ID\tSeq No\tStart\tEnd\n')
    for motif, posList in motifPos:
        for pos in posList:
            #Duplicated motif name as regex field in scanregex output
            fPos.write('%s\t%s\t%s\t%d\t%d\t%d\n' % ((motif,motif) + pos))
    fPos.close()

def printMotifCounts(iprFile, motifPos):
    print iprFile
    for motif, posList in motifPos: print '\t%s\t%d'%(motif,len(posList))

# main
if __name__ == '__main__': #allows import from interpreter
    iprPattern = '%s.iprscan'; posPattern = '%s.psltpos'
    if len(sys.argv) < 2: print USAGE; sys.exit(1)
    motifFile = sys.argv[1]
    if len(sys.argv) > 2: posPattern = sys.argv[2]
    if len(sys.argv) > 3: iprPattern = sys.argv[3]
    iprFiles = [iprPattern % comp for comp in COMPS]
    posFiles = [posPattern % comp for comp in COMPS]
    scanWriteAllIpr(iprFiles, posFiles, motifFile)
