#!/usr/bin/python
import re, time, sys, glob
from memecv import *

# Motifs are hard coded first
# Position file format:
#   NAME  REGEX   SEQID     SEQNO ST  END
#   nuc4  [KR]{4} NP_014710   3   17  21
#   nuc4  [KR]{4} null        3   17  21
# Note that END is the position (indexed from zero) of the first non-motif base,
#   end == st + len(motif)
#   motif == seq[st : end]

COMPS = ['YeastCyto', 'YeastER', 'YeastGolgi', 'YeastLyso', 'YeastMito',
         'YeastNuclear', 'YeastPero', 'YeastPM', 'YeastSecreted']

MOTIF_NAMES = [
    [],
    'hdel FxFxF gts a2b RRR kk1 rr1'.split(),
    'MxxE WW'.split(),
    'lys1 SLL gga1 cln3 WE'.split(),
    ['mit1'],
    'nuc4 largeT'.split(),
    'pts2 pex5'.split(),
    'gern NPxY YxRF kv1'.split(),
    []]
MOTIF_RE_STR = [
    [],
    '[KRHQSA][DENQ]EL$ F...F...F [RK].[RK].{0,19}$ F.{6}[IL][IL] RRR K.?K.{1,3}$ ^.{1,3}R.?R'.split(),
    'M..E WW'.split(),
    '[TIK]LP[LKI] SLL W..[FWY] M.{9}G WE'.split(),
    ['R.FS'],
    '[KR]{4} K[KR].[KR]'.split(),
    '[RK][LVIQ].....[HQ][LAF] [STAGCN][KRHM][LIVMAFY]$'.split(),
    '[(CC.)(C.C)(CC)]$ NP.Y Y.RF V..SL'.split(),
    []]

def scanMotifs(seqFilename, regex):
    fSeq = fastaInit(seqFilename)
    motifPos = [[] for i in range(len(regex))]
    iSeq = 0
    while True:
        (seqId, seq) = fastaReadNext(fSeq)
        if None == seq: break
        iSeq = iSeq + 1
        for i in range(len(regex)):
            m = regex[i].search(seq)
            if m: motifPos[i].append((seqId, iSeq, m.start(), m.end()))
    fSeq.close()
    return (motifPos, iSeq)

def writeMotPos(posFile, motifNames, motifReStr, motifPos):
    fPos = open(posFile, 'w')
    fPos.write('#Motif\tRegex\tSeq ID\tSeq No\tStart\tEnd\n')
    for i in range(len(motifNames)):
        for pos in motifPos[i]:
            fPos.write('%s\t%s\t%s\t%d\t%d\t%d\n' % (motifNames[i], \
                       motifReStr[i], pos[0], pos[1], pos[2], pos[3]))
    fPos.close()

def readMotPos(posFile):
    '''Read motif position file.
    Return format: [(name1, [(seqId,seqNo,st,end),(), ...]), (name2,insArr2)...]
    This function is for other modules (meme_motif) to call, not for this module.'''
    f = open(posFile)
    motNameInstances = []
    f.readline() # Column descriptions
    currName = ''
    while True:
        tokens = f.readline().split()
        if len(tokens) == 0: break
        if tokens[0] == currName:
            currIns.append((tokens[2],tokens[3],int(tokens[4]),int(tokens[5])))
        else:
            if currName!='': motNameInstances.append((currName, currIns))
            currName = tokens[0]
            currIns=[(tokens[2],tokens[3],int(tokens[4]),int(tokens[5]))]
    if currName!='': motNameInstances.append((currName, currIns))
    f.close()
    return motNameInstances

def printMotifCounts(seqFile, motifNames, motifReStr, motifPos, nSeq):
    print seqFile
    for i in range(len(motifNames)):
        print '\t%s\t%s\t%d/%d' % (motifNames[i], motifReStr[i], len(motifPos[i]), nSeq)

def scanWriteAll(seqFiles, posFiles, motifNames, motifReStr):
    '''Scan all sequence files with all corresponding motifs,
    and write to corresponding position files'''
    for j in range(len(seqFiles)):
        motifRegex = [re.compile(str) for str in motifReStr[j]]
        (motifPos, nSeq) = scanMotifs(seqFiles[j], motifRegex)
        writeMotPos(posFiles[j], motifNames[j], motifReStr[j], motifPos)
        printMotifCounts(seqFiles[j], motifNames[j], motifReStr[j], motifPos, nSeq)

# main
if __name__ == '__main__': #allows import from interpreter
    seqPattern = '~/subloc/data/hallett2004/%s.txt'
    if len(sys.argv) > 1: seqPattern = sys.argv[1]
    
    seqFiles = [seqPattern % comp for comp in COMPS]
    posFiles = ['%s.motpos' % comp for comp in COMPS]
    #print seqFiles
    #print posFiles
    scanWriteAll(seqFiles, posFiles, MOTIF_NAMES, MOTIF_RE_STR)
