import sys
from seq2cv import HALLETT05LABELS, loadLabeledFile, saveId2SeqAbbr
from seq2stat import removeSpace

USAGE = '''Usage:
seq2cv LABELED_FILE
    Convert LABELED_FILE into 9 compartment-specific FASTA files
    in the current directory, using predefined labels for hallett 2005 (yeast).
    ex. seq2comps.py ~/data/hallett2005/Hallett.out
'''

def generateCompFasta(labeledFileName, labelAbbr):
    abbrDict = dict([(abbr, 1) for abbr in labelAbbr.values()])
    abbrs = abbrDict.keys()
    print 'Loading sequence file...'
    (id2seqAbbr, skippedLabels, nSeq) = loadLabeledFile(labeledFileName, labelAbbr)
    for id, seqAbbrs in id2seqAbbr.items():
        id2seqAbbr[id] = (removeSpace(seqAbbrs[0]), seqAbbrs[1])
    print 'Saving FASTA files of each compartment...'
    saveId2SeqAbbr(id2seqAbbr, abbrs, 'Yeast%s.fa')

    print '%d files written' % len(abbrs)
    print '%d unique sequences' % nSeq
    print 'Skipped lables:'; print skippedLabels

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) <= 1: print USAGE; sys.exit(-1)
    labeledFileName = sys.argv[1]
    generateCompFasta(labeledFileName, HALLETT05LABELS)
