#!/usr/bin/env python
# Single-compartment version of seq2cv.py:
# Convert Murphy's labeled file format (separated by //) into FASTA format
# (into compartment and fold specific file),
# only creating single-compartment training sequences.

import sys, random
from seq2cv import HALLETT05LABELS, loadLabeledFile, saveId2SeqAbbr

USAGE = '''Usage:
seq2cv LABELED_FILE [SEED] [FOLDS]
    Convert LABELED_FILE into 2*FOLDS*9 FASTA files in the same directory,
    using predefined labels for hallett 2005 (yeast).
    Default values: SEED=1000, FOLDS=10.
    ex. seq2cv.py ~/data/hallett2005/Hallett.out
'''

def generateCvFasta(labeledFileName, labelAbbr, seed, nFold, prefix):
    'The code in this function is the same as in seq2cv'
    abbrDict = dict([(abbr, 1) for abbr in labelAbbr.values()])
    abbrs = abbrDict.keys()
    print 'Loading sequence file...'
    (id2seqAbbr, skippedLabels, nSeq) = loadLabeledFile(labeledFileName, labelAbbr)
    print 'Saving FASTA files of each fold...'
    saveCvFiles(id2seqAbbr, abbrs, seed, nFold, prefix)

    print '%d files written (%d folds, %d compartments)' \
          % (2 * nFold * len(abbrs), nFold, len(abbrs))
    print '%d unique sequences' % nSeq
    print 'Skipped lables:'; print skippedLabels
    print ''
    print 'Execute maketree.sh and then check_seq2cv.py to check!'

def saveCvFiles(id2seqAbbr, abbrs, seed, nFold, prefix):
    'The code in this function is similar to the function in seq2cv'
    (abbrUniqIds, multiCompIds) = seperateSingleMultiComp(id2seqAbbr, abbrs)
    random.seed(seed)
    for ids in abbrUniqIds.values(): random.shuffle(ids)
    for fold in range(nFold):
        id2seqAbbrTr = {}; id2seqAbbrTe = {}
        for ids in abbrUniqIds.values():
            start = len(ids)*fold/nFold
            end = len(ids)*(fold+1)/nFold
            for id in ids[:start]+ids[end:]: id2seqAbbrTr[id] = id2seqAbbr[id]
            for id in ids[start : end]     : id2seqAbbrTe[id] = id2seqAbbr[id]
        for id in multiCompIds: id2seqAbbrTe[id] = id2seqAbbr[id]
        saveId2SeqAbbr(id2seqAbbrTr,abbrs,'%s%%s%dtr%d.fa' % (prefix,seed,fold))
        saveId2SeqAbbr(id2seqAbbrTe,abbrs,'%s%%s%dte%d.fa' % (prefix,seed,fold))

def seperateSingleMultiComp(id2seqAbbr, abbrs):
    'This is the main difference from seq2cv'
    multiCompIds = []
    singleCompIds = dict([(abbr, []) for abbr in abbrs])
    for id, seqAbbrs in id2seqAbbr.items():
        (seq, abbrs1) = seqAbbrs
        if len(abbrs1) > 1: multiCompIds.append(id)
        else: singleCompIds[abbrs1[0]].append(id)
    return (singleCompIds, multiCompIds)

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) <= 1: print USAGE; sys.exit(-1)
    labeledFileName = sys.argv[1]
    seed = 1000
    if len(sys.argv) > 2: seed = int(sys.argv[2])
    nFold = 10
    if len(sys.argv) > 3: nFold = int(sys.argv[3])
    labelAbbr = HALLETT05LABELS
    outPrefix = 'Yeast'

    generateCvFasta(labeledFileName, labelAbbr, seed, nFold, outPrefix)
