#!/usr/bin/env python
# From Murphy's labeled sequence file format (separated by //),
# save single-compartment sequences into compartment-specific FASTA files.

import sys
from seq2cv import HALLETT05LABELS, loadLabeledFile, saveId2SeqAbbr

USAGE = '''Usage:
seq2single2all LABELED_FILE
    Convert LABELED_FILE into 9 FASTA files containing only single-compartment
    sequences. Labels are predefined for hallett 2005 (yeast).
    ex. seq2single2all.py ~/data/hallett2005/Hallett.out
'''

def generateFasta(labeledFileName, labelAbbr, prefix):
    'Similar to seq2cv.generateCvFasta().'
    abbrDict = dict([(abbr, 1) for abbr in labelAbbr.values()])
    abbrs = abbrDict.keys()
    print 'Loading sequence file...'
    (id2seqAbbr, skippedLabels, nSeq) = loadLabeledFile(labeledFileName, labelAbbr)
    id2seqAbbr1 = dict([(id, seqAbbrs) for id, seqAbbrs in id2seqAbbr.items()\
                        if len(seqAbbrs[1]) == 1])
    saveId2SeqAbbr(id2seqAbbr1, abbrs, prefix + '%s.fa')

    print '%d compartment-specific FASTA files written' % len(abbrs)
    print 'Skipped lables:'; print skippedLabels

# main
if __name__ == '__main__': #allows import from interpreter
    if len(sys.argv) <= 1: print USAGE; sys.exit(-1)
    labeledFileName = sys.argv[1]
    generateFasta(labeledFileName, HALLETT05LABELS, 'Yeast')
