#Show statistics of Hallett 2005 data format by default.
#Show statistics of Hallett 2004 data format when given more than one arguments.

import sys
from seq2cv import HALLETT05LABELS, loadLabeledFile
from memecv import sum, fastaReadAll

DEFAULT_LABELED_FILE = 'Hallett.out'

def showStatMultiComp(id2seqAbbr):
    nSeq = len(id2seqAbbr)
    maxNAbbr = max([len(seqAbbrs[1]) for id, seqAbbrs in id2seqAbbr.items()])
    histogram = [0] * maxNAbbr
    for id, seqAbbrs in id2seqAbbr.items():
        histogram[len(seqAbbrs[1])-1] = histogram[len(seqAbbrs[1])-1] + 1
    print '#comp\tcounts\tpercentage'
    for nAbbr in range(len(histogram)):
        print '%d\t%d\t%.0f%%' % (nAbbr + 1, histogram[nAbbr], \
                                  100. * histogram[nAbbr] / nSeq)

def showCompCooccurrence(id2seqAbbr, labelAbbr):
    abbrDict = dict([(abbr, 1) for abbr in labelAbbr.values()])
    abbrs = abbrDict.keys()
    abbrs.sort()

    count = [[0] * len(abbrs) for i in range(len(abbrs))]
    for id, seqAbbrs in id2seqAbbr.items():
        (seq, abbrs1) = seqAbbrs
        if len(abbrs1) == 2:
            countCooccurrence(count, abbrs, abbrs1[0], abbrs1[1])
        elif len(abbrs1) == 3:
            countCooccurrence(count, abbrs, abbrs1[0], abbrs1[1])
            countCooccurrence(count, abbrs, abbrs1[0], abbrs1[2])
            countCooccurrence(count, abbrs, abbrs1[1], abbrs1[2])
    print '\nCooccurrence:'
    print '         \t' + '\t'.join(abbrs)
    for rowAbbr, row in zip(abbrs, count):
        print ('%-9s\t' % rowAbbr) + '\t'.join([str(elem) for elem in row])

def countCooccurrence(count, abbrs, abbr1, abbr2):
    i1 = abbrs.index(abbr1); i2 = abbrs.index(abbr2)
    assert i1 != i2
    count[i1][i2] = count[i1][i2] + 1; count[i2][i1] = count[i2][i1] + 1

def showCompCooccur3(id2seqAbbr, labelAbbr):
    abbrDict = dict([(abbr, 1) for abbr in labelAbbr.values()])
    abbrs = abbrDict.keys()
    abbrs.sort()

    count3 = {}
    for id, seqAbbrs in id2seqAbbr.items():
        (seq, abbrs1) = seqAbbrs
        if len(abbrs1) == 3:
            abbrs1.sort()
            abbrs1t = tuple(abbrs1)
            if abbrs1t in count3: count3[abbrs1t] = count3[abbrs1t] + 1
            else: count3[abbrs1t] = 1
    count3list = count3.items()
    count3list.sort(lambda x,y: cmp(y[1], x[1]))
    print '\nTri-occurrences:'
    for abbrs1t, c in count3list:
        print '%-20s\t%d' % (('%s,%s,%s' % abbrs1t), c)

def showStatLen(id2seqAbbr):
    idLenList = [(id+' '+' '.join(seqAb[1]), len(removeSpace(seqAb[0]))) \
                 for id, seqAb in id2seqAbbr.items()]
    idLenList.sort(lambda x,y: cmp(y[1], x[1]))
    # Longest 50 proteins
    print '\nLongest proteins'
    for id1, len1 in idLenList[:50]: print '%d\t%s' % (len1, id1)
    # Medium and mean
    if len(idLenList) % 2 == 0:
        print 'Medium length: %d' % idLenList[len(idLenList)/2][1]
    else:
        print 'Medium length: %.1f' % (idLenList[len(idLenList)/2][1] \
                                       + idLenList[len(idLenList)/2+1][1]) / 2.0
    print 'Mean length: %.1f' % (sum([x[1] for x in idLenList])/len(idLenList))
    # Write length to CSV for Matlab to read
    f = open('seqlen.csv', 'w')
    for id1, len1 in idLenList: f.write('%d\n' % len1)
    f.close()

def removeSpace(seq):
    return ''.join([c for c in seq if c not in ' \t\n\r'])

def analizeCompFiles(compSeqFiles):
    idLenList = []
    for seqFile in compSeqFiles:
        idLenList = idLenList + [(idSeq[0]+' '+seqFile, len(idSeq[1])) \
                                 for idSeq in fastaReadAll(seqFile)]
    idLenList.sort(lambda x,y: cmp(y[1], x[1]))
    print '\nLongest proteins'
    for id1, len1 in idLenList[:50]: print '%d\t%s' % (len1, id1)
    f = open('allseqlen.csv', 'w')
    for id1, len1 in idLenList: f.write('%d\n' % len1)
    f.close()

# main
if __name__ == '__main__': #allows import from interpreter
    labeledFileName = DEFAULT_LABELED_FILE
    if len(sys.argv) == 2: labeledFileName = sys.argv[1]
    elif len(sys.argv) > 2: analizeCompFiles(sys.argv[1:]); sys.exit(0);
    
    (id2seqAbbr, skippedLabels, nSeq) = loadLabeledFile(labeledFileName, \
                                                        HALLETT05LABELS)
    if len(skippedLabels) > 0: print 'Skipped lables:'; print skippedLabels
    showStatMultiComp(id2seqAbbr)
    showCompCooccurrence(id2seqAbbr, HALLETT05LABELS)
    showCompCooccur3(id2seqAbbr, HALLETT05LABELS)
    showStatLen(id2seqAbbr)
