#Retrieve fungal alignments from SGD.
#Proteins to retrieve are given from stdin.
#Alignments are stored as HTML pages (skipped if exist).
#Run with sgd_download_job.py as:
#python sgd_download_job.py Hallett.out 19 | python sgd_download.py

import sys, os, urllib

def downloadByStream(f):
    '''Download HTML pages of all proteins given by the input stream
    (stdin or file).'''
    for line in f:
        protein = line[:-1] #remove \n
        downloadAlignment(protein, protein + '.html')

def downloadAlignment(protein, filename):
    'Download HTML page of a single protein, skip if file exists.'
    if os.path.exists(filename): return
    response = urllib.urlopen('http://db.yeastgenome.org/fungi/%s.html' % protein)
    f = open(filename, 'w')
    f.write(response.read()) #not that large; typically less than 1MB
    f.close()
    response.close()

def downloadAlignmentCGI(protein, filename):
    'Invoke SGD showAlign CGI script by HTTP POST (for doc and archive)'
    url = 'http://db.yeastgenome.org/cgi-bin/FUNGI/showAlign'
    data = {'locus': protein}
    response = urllib2.urlopen(urllib2.Request(url, urllib.urlencode(data)))
    f = open(filename, 'w')
    line = response.readline()
    while len(line) > 0:
        f.write(line)
        line = response.readline()
    f.close()
    response.close()

# main
if __name__ == '__main__': #allows import from interpreter
    downloadByStream(sys.stdin)
